using System;
using MapPointLibrary.MapPointService;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using ADS.MapPoint.Proxies;

namespace ADS.MapPoint
{
	public class Cartographer
	{
		// hold our location
		public Pushpin LastLocation;
		public string cachePath;

		public Cartographer()
		{
			LastLocation = new Pushpin();
			LastLocation.LatLong = new LatLongCoord();
		}


		public void SaveMapImage(double Latitude, double Longitude, int ImageHeight, int ImageWidth, int ZoomLevel, string FileName)
		{
			RenderProxy renderProxy = new RenderProxy();
			MapImage mapImage = new MapImage();

			try
			{
				mapImage = renderProxy.GetMap(Latitude.ToString(), Longitude.ToString(), ImageHeight, ImageWidth, ZoomLevel, LastLocation);
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Failed to GetCenterPoint");
			}
			
			MIMEData mimeData = mapImage.Image;
			
			MapView mapView = mapImage.View;

			MemoryStream ms = new MemoryStream(mimeData.Bits);
			FileStream fs = new FileStream(FileName, FileMode.Create);
			
			try
			{
				// first write header
				Debugging.DEBUGMSG(true, "Writing Lat: " + Latitude.ToString() + "\r\n");
				fs.Write(System.BitConverter.GetBytes(Latitude), 0, 8);
				Debugging.DEBUGMSG(true, "Writing Lon: " + Longitude.ToString() + "\r\n");
				fs.Write(System.BitConverter.GetBytes(Longitude), 0, 8);

				// next write image
				byte[] read = new byte[256];
				int count = ms.Read(read, 0, 256);

				while(count > 0) 
				{
					fs.Write(read, 0, count);
					count = ms.Read(read, 0, 256);
				}

				fs.Close();
				ms.Close();
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "Display failure: " + ex.Message + "\r\n");
			}
		}

		public void DisplayMapImage(string FileName, PictureBox Destination, bool FileHasHeader)
		{
			Bitmap bitmap = null;

			if(FileHasHeader)
			{
				try
				{
					byte[] latlon = new Byte[16];
					byte[] img = new Byte[256];

					FileStream fsin = new FileStream(FileName, FileMode.Open, FileAccess.Read);
					FileStream fsout = new FileStream(cachePath + @"\temp.gif", FileMode.Create);

					fsin.Read(latlon, 0, 16);
				
					// store in LastLocation
					LastLocation.LatLong.Latitude = System.BitConverter.ToDouble(latlon, 0);
					Debugging.DEBUGMSG(true, "Reading Lat: " + LastLocation.LatLong.Latitude.ToString() + "\r\n");
					LastLocation.LatLong.Longitude = System.BitConverter.ToDouble(latlon, 8);
					Debugging.DEBUGMSG(true, "Reading Lon: " + LastLocation.LatLong.Longitude.ToString() + "\r\n");

					int count = fsin.Read(img, 0, 256);
//					Debugging.DEBUGMSG(true, "Prime: read " + count + " bytes\r\n");

					while(count > 0)
					{
						fsout.Write(img, 0, count);
//						Debugging.DEBUGMSG(true, "wrote " + count + " bytes\r\n");
						count = fsin.Read(img, 0, img.GetLength(0));
//						Debugging.DEBUGMSG(true, "read " + count + " bytes\r\n");
					}
					Debugging.DEBUGMSG(true, "closing files...\r\n");
					fsin.Close();
					fsout.Close();
					Debugging.DEBUGMSG(true, "closed.\r\n");
				}
				catch(Exception ex)
				{
					Debugging.DEBUGMSG(true, "Display failure: " + ex.Message + "\r\n");
				}

				FileName = cachePath + @"\temp.gif";
			}
			
			Debugging.DEBUGMSG(true, "loading bitmap from " + FileName + "...\r\n");
			try
			{
				bitmap = new Bitmap(FileName);
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "failed loading bitmap: " + ex.Message + "\r\n");
			}

			Debugging.DEBUGMSG(true, "displayinging bitmap...\r\n");
			Destination.Image = bitmap;
		}
		
		public void DisplayMapImage(double Latitude, double Longitude, int ZoomLevel, PictureBox Destination, bool CacheMap)
		{	
			string fileName = "";
			
			if(CacheMap)
			{
				string[] files = null;

				try
				{					
					int high = 0;
					int current;

					files = Directory.GetFiles(cachePath, LastLocation.Label + "???.map");
					
					foreach(string file in files)
					{

						current = Convert.ToInt16(file.Substring(cachePath.Length + 1 + LastLocation.Label.Length, 3));
						if(current > high)
							high = current;
					}
					high++;
					fileName = "\\" + LastLocation.Label + high.ToString("000") + ".map";

				}
				catch(IOException)
				{
					fileName = "\\" + LastLocation.Label + "001.map";
				}
				catch(Exception ex)
				{
					Debugging.DEBUGMSG(true, "Failed to get cache files: " + ex.Message + "\r\n");
				}				
			}
			else
			{
				fileName = "\\Last Map.map";
			}
			
			fileName = cachePath + fileName;
			
			SaveMapImage(Latitude, Longitude, Destination.Height, Destination.Width, ZoomLevel, fileName);
			DisplayMapImage(fileName, Destination, true);
		}
		
		public void GeocodeAddress(ref Address Address)
		{
			FindProxy findProxy = new FindProxy();
			try
			{
				Debugging.DEBUGMSG(true, "Sending address to MapPoint server...\r\n");
				FindResult findResult = findProxy.FindLatLongFromAddress(Address.Street, Address.City, Address.State, Address.Zip);

				Address.Latitude = findResult.FoundLocation.LatLong.Latitude;
				Address.Longitude = findResult.FoundLocation.LatLong.Longitude;

				Debugging.DEBUGMSG(true, "Lat: " + Address.Latitude + "\r\n");
				Debugging.DEBUGMSG(true, "Lon: " + Address.Longitude + "\r\n");

				Address.Geocoded = true;
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "No match found.\r\n");
				MessageBox.Show(ex.Message, "Failed to find address!");
			}
		}

		public double DistanceInKilometers(Address Address1, Address Address2)
		{
			CommonProxy commonProxy = new CommonProxy();
			double result = -1;

			LatLongCoord LatLong1 = new LatLongCoord();
			LatLongCoord LatLong2 = new LatLongCoord();

			LatLong1.Latitude = Address1.Latitude;
			LatLong1.Longitude = Address1.Longitude;
			LatLong2.Latitude = Address2.Latitude;
			LatLong2.Longitude = Address2.Longitude;

			Debugging.DEBUGMSG(true, "Getting distance between points:\r\n");
			Debugging.DEBUGMSG(true, "  " + Address1.Latitude + ", " + Address1.Longitude + "\r\n");
			Debugging.DEBUGMSG(true, "  " + Address2.Latitude + ", " + Address2.Longitude + "\r\n");

			try
			{
				result = commonProxy.GetGreatCircleDistance(LatLong1, LatLong2);
				Debugging.DEBUGMSG(true, "  Distance (km): " + result + "\r\n");
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "  Failed to get distance: " + ex.Message + "\r\n");
			}

			return result;
		}

		public double DistanceInMiles(Address Address1, Address Address2)
		{
			double result =	DistanceInKilometers(Address1, Address2) * 0.621371;

			Debugging.DEBUGMSG(true, "  Distance (mi): " + result + "\r\n");

			return result;
		}

		public void DisplayMapImage(ref Address AddressToMap, int ZoomLevel, PictureBox Destination)
		{
			GeocodeAddress(ref AddressToMap);

			SaveMapImage(AddressToMap.Latitude, AddressToMap.Longitude, Destination.Height, Destination.Width, ZoomLevel, "Map.map");
			DisplayMapImage("Map.map", Destination, true);
		}
	}
}
