using System;

namespace ADS.MapPoint
{
	#region ------------- Enumerations -------------
	public enum CoordinateDirection : short
	{
		North = 0,
		South,
		East,
		West
	}

	public enum CoordinateType : short
	{
		Latitude = 0,
		Longitude
	}
	#endregion

	#region ------------- Coordinate Class -------------
	public class Coordinate
	{
		private int m_degrees;
		private int m_minutes;
		private double m_seconds;
		private CoordinateType m_type;
		private CoordinateDirection m_direction;

		public Coordinate(CoordinateType Type)
		{
			m_type = Type;
		}

		#region --- Properties---
		
		public int Degrees
		{
			set
			{
				// check for valid value
				if(m_type == CoordinateType.Latitude)
				{
					if((value > 90) || (value < 0))
					{
						throw new Exception("Latitude degrees must be between 0 and 90");
					}
				}
				else
				{
					if((value > 180) || (value < 0))
					{
						throw new Exception("Longitude degrees must be between 0 and 180");
					}
				}

				// store value
				m_degrees = value;
			}
			get
			{
				return m_degrees;
			}
		}

		public int Minutes
		{
			set
			{
				// check for valid value
				if((value > 59) || (value < 0))
				{
					throw new Exception("Minutes must be between 0 and 59");
				}
				
				// store value
				m_minutes = value;
			}
			get
			{
				return m_minutes;
			}
		}

		public double Seconds
		{
			set
			{
				// check for valid value
				if((value > 59) || (value < 0))
				{
					throw new Exception("Minutes must be between 0 and 59");
				}

				// store value
				m_seconds = value;
			}
			get
			{
				return m_seconds;
			}
		}

		public CoordinateDirection Direction
		{
			set
			{
				// check for valid value
				if(m_type == CoordinateType.Latitude)
				{
					if(value > CoordinateDirection.South)
					{
						throw new Exception("Latitude Direction must be North or South");
					}
				}
				else
				{
					if(value < CoordinateDirection.East)
					{
						throw new Exception("Longitude Direction must be East or West");
					}
				}

				// store value
				m_direction = value;
			}
			get
			{
				return m_direction;
			}
		}
		#endregion

		#region operator overloads, conversions
		// cast to a float
		public static implicit operator double(Coordinate operand)
		{
			double retval;

			retval = operand.Degrees;
			retval += (((double)operand.Minutes) / 60);
			retval += (operand.Seconds / 3600);

			if((operand.Direction == CoordinateDirection.South) || (operand.Direction == CoordinateDirection.West))
			{
				retval *= -1;
			}

			return retval;
		}
				
		// convert current value to a decimal
		public void FromDouble(double Value)
		{
			double mins;

			if(Value < 0)
			{
				if(this.m_type == CoordinateType.Latitude)
				{
					this.m_direction = CoordinateDirection.South;
				}
				else
				{
					this.m_direction = CoordinateDirection.West;
				}
			}
			else
			{
				if(this.m_type == CoordinateType.Latitude)
				{
					this.m_direction = CoordinateDirection.North;
				}
				else
				{
					this.m_direction = CoordinateDirection.East;
				}
			}

			this.Degrees = (int)Math.Floor(Math.Abs(Value)); 

			mins = (double)(Math.Abs(Value) - this.Degrees) * 60;
			this.Minutes = (int)Math.Floor(mins);

			this.Seconds = (double)(mins - this.Minutes) * 60;
		}

		public override string ToString()
		{
			// return a string in format:  DD MM' SS.SS" N(S,E,W)
			string retval;

			retval = this.Degrees.ToString() + " ";
			retval += (this.Minutes.ToString() + "' ");
			retval += (string.Format("#.##", this.Seconds) + "\" ");
			
			switch(this.Direction)
			{
				case CoordinateDirection.East:
					retval += "E";
					break;
				case CoordinateDirection.North:
					retval += "N";
					break;
				case CoordinateDirection.South:
					retval += "S";
					break;
				case CoordinateDirection.West:
					retval += "W";
					break;
			}

			return retval;
		}
		#endregion // operator overloads
	}
	#endregion  // Coordinate class

	#region ------------- CoordinateSet Class -------------
	public class CoordinateSet
	{
		private Coordinate m_latitude;
		private Coordinate m_longitude;

		public CoordinateSet()
		{
			m_latitude = new Coordinate(CoordinateType.Latitude);
			m_longitude = new Coordinate(CoordinateType.Longitude);
		}

		public CoordinateSet(Coordinate Latitude, Coordinate Longitude)
		{
			m_latitude = Latitude;
			m_longitude = Longitude;
		}

		public Coordinate Latitude
		{
			set
			{
				// value checking done by Coordinate class
				m_latitude = value;
			}
			get
			{
				return m_latitude;
			}
		}

		public Coordinate Longitude
		{
			set
			{
				// value checking done by Coordinate class
				m_longitude = value;
			}
			get
			{
				return m_longitude;
			}
		}
	}
	#endregion
}
