using System;
using System.Net;
using MapPointLibrary.MapPointService;

namespace ADS.MapPoint.Proxies
{
	#region --------------- FindProxy ---------------
	internal class FindProxy
	{
		FindServiceSoap FindService = null;

		static public string UserName = "[user]";
		static public string Password = "[password]";


		public FindProxy()
		{
			FindService = new FindServiceSoap();
			FindService.Credentials = new NetworkCredential(UserName, Password);
		}

		public FindResult FindLatLongFromAddress(string street, string city, string state, string zip)
		{
			FindResult[] address = null; 
			DataSource FindServiceDataSource = new DataSource();
			FindServiceDataSource.Name = "MapPoint.NA";
			FindOptions FindOptions  = new FindOptions();
			FindOptions.GeoDataSource = FindServiceDataSource;
			FindOptions.ResultMask = FindResultMask.LatLongFlag;

			MapPointLibrary.MapPointService.Address FindServiceAddress = new MapPointLibrary.MapPointService.Address();
			FindServiceAddress.AddressLine = street;
			FindServiceAddress.PrimaryCity = city;
			FindServiceAddress.Subdivision = state;
			FindServiceAddress.PostalCode = zip;
			FindServiceAddress.CountryRegion = "USA"; 

			int numFound = 0;

			try
			{
				address = FindService.FindAddress(FindServiceAddress, FindOptions ,out numFound);
			}
			catch(Exception)
			{
				//System.Windows.Forms.MessageBox.Show(ex.Message, "Error");
			}

			if(numFound > 0)
			{
				return address[0];
			}
			else
			{
				return null;
			}
		}
	}
	#endregion

	#region --------------- RenderProxy ---------------
	internal class RenderProxy
	{
		RenderServiceSoap RenderService = null;
		static public string UserName = "[user]";
		static public string Password = "[password]";

		public RenderProxy()
		{
			RenderService = new RenderServiceSoap();
			RenderService.Credentials = new NetworkCredential(UserName, Password);
		}

		public MapImage GetMap(string latPoint, string longPoint, int ImageHeight, int ImageWidth, int ZoomLevel, Pushpin location)
		{
			// Create and specify MapView object, set centerpoint and scale.
			MapView MapView = new MapView();
			MapView.CenterPoint = new LatLongCoord();
			MapView.CenterPoint.Latitude = System.Convert.ToDouble(latPoint);
			MapView.CenterPoint.Longitude = System.Convert.ToDouble(longPoint);
			MapView.Scale = ZoomLevel;

			// Create an array of MapView objects, insert previously created single MapView object.
			MapView[] MapViews = new MapView[1]; 
			MapViews[0] = MapView;

			// Create and specify MapOptions object, based on form and session data.
			MapOptions MapOptions = new MapOptions();
			MapOptions.Format = new ImageFormat();
			MapOptions.Format.Height = System.Convert.ToInt32(ImageHeight); 
			MapOptions.Format.Width = System.Convert.ToInt32(ImageWidth); 
			
			// Create an array of PushPin objects, store a single PushPin in element 0.
			Pushpin[] PushPins = new Pushpin[1];
			PushPins[0] = location;

			// Creating the MapImage object and actually rendering the map.
			MapImage[] MapImages = new MapImage[1];

			MapImages = RenderService.GetMap(MapViews, MapOptions, PushPins);

			return MapImages[0];
		}
	}
	#endregion

	#region --------------- CommonProxy ---------------
	internal class CommonProxy
	{
		CommonServiceSoap CommonService = null;

		static public string UserName = "[user]";
		static public string Password = "[password]";


		public CommonProxy()
		{
			CommonService = new CommonServiceSoap();
			CommonService.Credentials = new NetworkCredential(UserName, Password);
		}

		public double GetGreatCircleDistance(LatLongCoord LatLong1, LatLongCoord LatLong2)
		{
			double[] results = new Double[1];
			LatLongCoord[] points = new LatLongCoord[2];
			
			points[0] = new LatLongCoord();
			points[1] = new LatLongCoord();

			points[0].Latitude = LatLong1.Latitude;
			points[0].Longitude = LatLong1.Longitude;
			points[1].Latitude = LatLong2.Latitude;
			points[1].Longitude = LatLong2.Longitude;

			results = CommonService.GetGreatCircleDistances(points);

			return results[0];
		}
	}
	#endregion
}
