using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.Net;
using System.Runtime.InteropServices;
using System.IO;
using ADS;
using ADS.MapPoint;
using System.Threading;

namespace TruckTrack
{
	public enum PanDirection
	{
		North,
		South,
		East,
		West
	}


	public class MainForm : System.Windows.Forms.Form
	{
		static void Main() 
		{
			Debugging.DEBUGMSG(true, "---- App Starting ----\n\r");			

			if(!Tools.IsInstanceRunning())
			{
				GlobalProxySelection.Select = GlobalProxySelection.GetEmptyWebProxy();

				Application.Run(new MainForm());
			}
			Debugging.DEBUGMSG(true, "---- App Ending ----\n\r");
		}

		#region --- member variables ---
		private double			m_currentLatitude;
		private double			m_currentLongitude;
		private double			mapCenterLatitude;
		private double			mapCenterLongitude;
		private string			pointLabel;
		private short			m_currentZoom;
		private Address			m_currentAddress;
		private	VehicleData		m_vehicleData;

		private int				m_gpsReadInterval;
		private Cartographer	cartographer		= new Cartographer();
		private ADSToBaltimore	simulationData		= new ADSToBaltimore();
		private bool			simulationForward	= true;
		
		private System.Windows.Forms.Timer simulationTimer;
		private System.Windows.Forms.Timer dataTimer;
		#endregion

		#region --- UI Variables ---
		private System.Windows.Forms.TabPage cacheTab;
		private System.Windows.Forms.TabPage mapTab;
		private System.Windows.Forms.TabPage findTab;
		private System.Windows.Forms.TabControl formTabs;
		private System.Windows.Forms.PictureBox mapPicture;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TabPage meTab;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.TextBox findState;
		private System.Windows.Forms.TextBox findCity;
		private System.Windows.Forms.TextBox findAddress;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label latitudeLabel;
		private System.Windows.Forms.Label longitudeLabel;
		private System.Windows.Forms.ListView cacheList;
		private System.Windows.Forms.Label label11;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label label9;
		private System.Windows.Forms.Button panEast;
		private System.Windows.Forms.Button panWest;
		private System.Windows.Forms.Button panSouth;
		private System.Windows.Forms.Button panNorth;
		private System.Windows.Forms.Button zoomOut;
		private System.Windows.Forms.Button zoomIn;
		private System.Windows.Forms.Button btnFind;
		private System.Windows.Forms.Button btnShowCached;
		private System.Windows.Forms.Button btnClearChecked;
		private System.Windows.Forms.Button btnClearCache;
		private System.Windows.Forms.TextBox txtMyLongitude;
		private System.Windows.Forms.TextBox txtMyLatitude;
		private System.Windows.Forms.Label label10;
		private System.Windows.Forms.CheckBox chkSimulateGPS;
		private System.Windows.Forms.TabPage dataTab;
		private System.Windows.Forms.Label label13;
		private System.Windows.Forms.Button btnClearData;
		private System.Windows.Forms.Button btnUploadData;
		private System.Windows.Forms.ListView DataListView;
		private System.Windows.Forms.TabPage optionsTab;
		private System.Windows.Forms.TextBox txtGPSInterval;
		private System.Windows.Forms.Label label14;
		private System.Windows.Forms.TextBox txtServerInterval;
		private System.Windows.Forms.Label label15;
		private System.Windows.Forms.Label label16;
		private System.Windows.Forms.Label label17;
		private System.Windows.Forms.Button btnApplyOptions;
		private System.Windows.Forms.ListBox lstActivity;
		private System.Windows.Forms.Label label12;
		private System.Windows.Forms.Label lblDistance;
		private System.Windows.Forms.Label label19;
		private System.Windows.Forms.Label label20;
		private System.Windows.Forms.Label mapStatus;
		#endregion
		
		#region --- Properties ---
		private int GPSReadInterval
		{
			set
			{
				m_gpsReadInterval = value;
				
				simulationTimer.Interval = m_gpsReadInterval;

				txtGPSInterval.Text = ((long)(m_gpsReadInterval / 1000)).ToString();

				lstActivity.Items.Insert(0, "GPS read interval set to " + ((long)(m_gpsReadInterval / 1000)).ToString());
			}
			get
			{
				return m_gpsReadInterval;
			}
		}

		private int DataSendInterval
		{
			set
			{
				dataTimer.Enabled = false;
				dataTimer.Interval = value;
				dataTimer.Enabled = true;

				txtServerInterval.Text = ((long)(dataTimer.Interval / 1000)).ToString();
				
				lstActivity.Items.Insert(0, "Send interval set to " + ((long)(dataTimer.Interval / 1000)).ToString());
			}
			get
			{
				return dataTimer.Interval;
			}
		}

		private double CurrentLatitude
		{
			set
			{
				m_currentLatitude = value;

				txtMyLatitude.Text = m_currentLatitude.ToString();

				lstActivity.Items.Insert(0, "Latitude is now " + m_currentLatitude.ToString());
			}
			get
			{
				return m_currentLatitude;
			}
		}

		private double CurrentLongitude
		{
			set
			{
				m_currentLongitude = value;

				txtMyLongitude.Text = m_currentLongitude.ToString();
				
				lstActivity.Items.Insert(0, "Longitude is now " + m_currentLongitude.ToString());
			}
			get
			{
				return m_currentLongitude;
			}
		}

		#endregion

		#region --- form ctor, dtor and events ---
		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// create any needed objects
			m_currentAddress = new Address();

			// create event handlers
			panNorth.Click += new EventHandler(PanZoom_Click);
			panSouth.Click += new EventHandler(PanZoom_Click);
			panEast.Click += new EventHandler(PanZoom_Click);
			panWest.Click += new EventHandler(PanZoom_Click);
			zoomIn.Click += new EventHandler(PanZoom_Click);
			zoomOut.Click += new EventHandler(PanZoom_Click);
			formTabs.SelectedIndexChanged += new EventHandler(TabClick);

			// get the map cache path
			cartographer.cachePath = "\\mapcache";
			if(!System.IO.Directory.Exists(cartographer.cachePath))
				System.IO.Directory.CreateDirectory(cartographer.cachePath);
						
			cacheList.CheckBoxes = true;
			cacheList.Columns.Add("Map", cacheList.Width, HorizontalAlignment.Left);
			
			// set a default data point for demo purposes
			findAddress.Text = "300 W Pratt";
			findCity.Text = "Baltimore";
			findState.Text = "MD";

			// create our timers
			simulationTimer = new System.Windows.Forms.Timer();
			simulationTimer.Enabled = false;
			simulationTimer.Tick += new EventHandler(simulationTimer_Tick);
			
			dataTimer = new System.Windows.Forms.Timer();
			dataTimer.Tick += new EventHandler(dataTimer_Tick);

			// get a unique VIN and initialize our data
			m_vehicleData = new VehicleData(GetVIN());

			// set defaults
			m_currentZoom = 1;
			GPSReadInterval = 15000;
			DataSendInterval = 60000;

			// start the timers
			dataTimer.Enabled = true;

			// reorder the tabs since it's a CF bug
			formTabs.Controls.SetChildIndex(this.mapTab, 1);
			formTabs.Controls.SetChildIndex(this.findTab, 1);
			formTabs.Controls.SetChildIndex(this.dataTab, 1);
			formTabs.Controls.SetChildIndex(this.cacheTab, 1);

			// load any existing data
			m_vehicleData.Load();
		}

		protected override void OnClosing(System.ComponentModel.CancelEventArgs args)
		{
			// save any unsent data
			m_vehicleData.Save();
		}

		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.formTabs = new System.Windows.Forms.TabControl();
			this.meTab = new System.Windows.Forms.TabPage();
			this.label12 = new System.Windows.Forms.Label();
			this.lstActivity = new System.Windows.Forms.ListBox();
			this.chkSimulateGPS = new System.Windows.Forms.CheckBox();
			this.label10 = new System.Windows.Forms.Label();
			this.txtMyLongitude = new System.Windows.Forms.TextBox();
			this.label4 = new System.Windows.Forms.Label();
			this.txtMyLatitude = new System.Windows.Forms.TextBox();
			this.label5 = new System.Windows.Forms.Label();
			this.cacheTab = new System.Windows.Forms.TabPage();
			this.btnClearCache = new System.Windows.Forms.Button();
			this.btnClearChecked = new System.Windows.Forms.Button();
			this.btnShowCached = new System.Windows.Forms.Button();
			this.cacheList = new System.Windows.Forms.ListView();
			this.mapTab = new System.Windows.Forms.TabPage();
			this.label20 = new System.Windows.Forms.Label();
			this.lblDistance = new System.Windows.Forms.Label();
			this.label19 = new System.Windows.Forms.Label();
			this.label9 = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.panEast = new System.Windows.Forms.Button();
			this.panWest = new System.Windows.Forms.Button();
			this.panSouth = new System.Windows.Forms.Button();
			this.panNorth = new System.Windows.Forms.Button();
			this.zoomOut = new System.Windows.Forms.Button();
			this.zoomIn = new System.Windows.Forms.Button();
			this.mapStatus = new System.Windows.Forms.Label();
			this.label11 = new System.Windows.Forms.Label();
			this.longitudeLabel = new System.Windows.Forms.Label();
			this.latitudeLabel = new System.Windows.Forms.Label();
			this.label7 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.mapPicture = new System.Windows.Forms.PictureBox();
			this.findTab = new System.Windows.Forms.TabPage();
			this.btnFind = new System.Windows.Forms.Button();
			this.findState = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.findCity = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.findAddress = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.dataTab = new System.Windows.Forms.TabPage();
			this.btnUploadData = new System.Windows.Forms.Button();
			this.btnClearData = new System.Windows.Forms.Button();
			this.label13 = new System.Windows.Forms.Label();
			this.DataListView = new System.Windows.Forms.ListView();
			this.optionsTab = new System.Windows.Forms.TabPage();
			this.btnApplyOptions = new System.Windows.Forms.Button();
			this.label16 = new System.Windows.Forms.Label();
			this.label17 = new System.Windows.Forms.Label();
			this.txtServerInterval = new System.Windows.Forms.TextBox();
			this.label15 = new System.Windows.Forms.Label();
			this.txtGPSInterval = new System.Windows.Forms.TextBox();
			this.label14 = new System.Windows.Forms.Label();
			// 
			// formTabs
			// 
			this.formTabs.Controls.Add(this.meTab);
			this.formTabs.Controls.Add(this.cacheTab);
			this.formTabs.Controls.Add(this.mapTab);
			this.formTabs.Controls.Add(this.findTab);
			this.formTabs.Controls.Add(this.dataTab);
			this.formTabs.Controls.Add(this.optionsTab);
			this.formTabs.Location = new System.Drawing.Point(4, 4);
			this.formTabs.SelectedIndex = 0;
			this.formTabs.Size = new System.Drawing.Size(580, 380);
			// 
			// meTab
			// 
			this.meTab.Controls.Add(this.label12);
			this.meTab.Controls.Add(this.lstActivity);
			this.meTab.Controls.Add(this.chkSimulateGPS);
			this.meTab.Controls.Add(this.label10);
			this.meTab.Controls.Add(this.txtMyLongitude);
			this.meTab.Controls.Add(this.label4);
			this.meTab.Controls.Add(this.txtMyLatitude);
			this.meTab.Controls.Add(this.label5);
			this.meTab.Location = new System.Drawing.Point(4, 22);
			this.meTab.Size = new System.Drawing.Size(572, 354);
			this.meTab.Text = "Me";
			// 
			// label12
			// 
			this.label12.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label12.Location = new System.Drawing.Point(296, 4);
			this.label12.Size = new System.Drawing.Size(192, 24);
			this.label12.Text = "Activity Log";
			this.label12.TextAlign = System.Drawing.ContentAlignment.TopCenter;
			// 
			// lstActivity
			// 
			this.lstActivity.Location = new System.Drawing.Point(292, 36);
			this.lstActivity.Size = new System.Drawing.Size(200, 264);
			// 
			// chkSimulateGPS
			// 
			this.chkSimulateGPS.Location = new System.Drawing.Point(56, 112);
			this.chkSimulateGPS.Size = new System.Drawing.Size(136, 20);
			this.chkSimulateGPS.Text = "Simulate GPS";
			this.chkSimulateGPS.CheckStateChanged += new System.EventHandler(this.chkSimulateGPS_CheckStateChanged);
			// 
			// label10
			// 
			this.label10.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label10.Location = new System.Drawing.Point(40, 8);
			this.label10.Size = new System.Drawing.Size(192, 24);
			this.label10.Text = "My Current Location";
			this.label10.TextAlign = System.Drawing.ContentAlignment.TopCenter;
			// 
			// txtMyLongitude
			// 
			this.txtMyLongitude.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.txtMyLongitude.Location = new System.Drawing.Point(112, 72);
			this.txtMyLongitude.Size = new System.Drawing.Size(132, 26);
			this.txtMyLongitude.Text = "";
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label4.Location = new System.Drawing.Point(20, 72);
			this.label4.Size = new System.Drawing.Size(88, 20);
			this.label4.Text = "Longitude:";
			this.label4.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// txtMyLatitude
			// 
			this.txtMyLatitude.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.txtMyLatitude.Location = new System.Drawing.Point(112, 40);
			this.txtMyLatitude.Size = new System.Drawing.Size(132, 26);
			this.txtMyLatitude.Text = "";
			// 
			// label5
			// 
			this.label5.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label5.Location = new System.Drawing.Point(36, 44);
			this.label5.Size = new System.Drawing.Size(72, 20);
			this.label5.Text = "Latitude:";
			this.label5.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// cacheTab
			// 
			this.cacheTab.Controls.Add(this.btnClearCache);
			this.cacheTab.Controls.Add(this.btnClearChecked);
			this.cacheTab.Controls.Add(this.btnShowCached);
			this.cacheTab.Controls.Add(this.cacheList);
			this.cacheTab.Location = new System.Drawing.Point(4, 22);
			this.cacheTab.Size = new System.Drawing.Size(572, 354);
			this.cacheTab.Text = "History";
			// 
			// btnClearCache
			// 
			this.btnClearCache.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Regular);
			this.btnClearCache.Location = new System.Drawing.Point(216, 120);
			this.btnClearCache.Size = new System.Drawing.Size(88, 40);
			this.btnClearCache.Text = "Clear All";
			this.btnClearCache.Click += new System.EventHandler(this.btnClearCache_Click);
			// 
			// btnClearChecked
			// 
			this.btnClearChecked.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Regular);
			this.btnClearChecked.Location = new System.Drawing.Point(216, 72);
			this.btnClearChecked.Size = new System.Drawing.Size(88, 40);
			this.btnClearChecked.Text = "Clear Checked";
			this.btnClearChecked.Click += new System.EventHandler(this.btnClearChecked_Click);
			// 
			// btnShowCached
			// 
			this.btnShowCached.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Regular);
			this.btnShowCached.Location = new System.Drawing.Point(216, 16);
			this.btnShowCached.Size = new System.Drawing.Size(88, 40);
			this.btnShowCached.Text = "Show Map";
			this.btnShowCached.Click += new System.EventHandler(this.btnShowCached_Click);
			// 
			// cacheList
			// 
			this.cacheList.FullRowSelect = true;
			this.cacheList.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.cacheList.Location = new System.Drawing.Point(8, 8);
			this.cacheList.Size = new System.Drawing.Size(200, 304);
			this.cacheList.View = System.Windows.Forms.View.Details;
			// 
			// mapTab
			// 
			this.mapTab.Controls.Add(this.label20);
			this.mapTab.Controls.Add(this.lblDistance);
			this.mapTab.Controls.Add(this.label19);
			this.mapTab.Controls.Add(this.label9);
			this.mapTab.Controls.Add(this.label8);
			this.mapTab.Controls.Add(this.panEast);
			this.mapTab.Controls.Add(this.panWest);
			this.mapTab.Controls.Add(this.panSouth);
			this.mapTab.Controls.Add(this.panNorth);
			this.mapTab.Controls.Add(this.zoomOut);
			this.mapTab.Controls.Add(this.zoomIn);
			this.mapTab.Controls.Add(this.mapStatus);
			this.mapTab.Controls.Add(this.label11);
			this.mapTab.Controls.Add(this.longitudeLabel);
			this.mapTab.Controls.Add(this.latitudeLabel);
			this.mapTab.Controls.Add(this.label7);
			this.mapTab.Controls.Add(this.label6);
			this.mapTab.Controls.Add(this.mapPicture);
			this.mapTab.Location = new System.Drawing.Point(4, 22);
			this.mapTab.Size = new System.Drawing.Size(572, 354);
			this.mapTab.Text = "Map";
			// 
			// label20
			// 
			this.label20.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Bold);
			this.label20.Location = new System.Drawing.Point(328, 328);
			this.label20.Size = new System.Drawing.Size(96, 24);
			this.label20.Text = "miles";
			// 
			// lblDistance
			// 
			this.lblDistance.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Bold);
			this.lblDistance.Location = new System.Drawing.Point(208, 328);
			this.lblDistance.Size = new System.Drawing.Size(112, 23);
			this.lblDistance.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label19
			// 
			this.label19.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Bold);
			this.label19.Location = new System.Drawing.Point(24, 328);
			this.label19.Size = new System.Drawing.Size(168, 24);
			this.label19.Text = "Distance From You:";
			this.label19.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label9
			// 
			this.label9.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label9.Location = new System.Drawing.Point(432, 256);
			this.label9.Size = new System.Drawing.Size(64, 24);
			this.label9.Text = "Zoom";
			this.label9.TextAlign = System.Drawing.ContentAlignment.TopCenter;
			// 
			// label8
			// 
			this.label8.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label8.Location = new System.Drawing.Point(496, 256);
			this.label8.Size = new System.Drawing.Size(72, 24);
			this.label8.Text = "Pan";
			this.label8.TextAlign = System.Drawing.ContentAlignment.TopCenter;
			// 
			// panEast
			// 
			this.panEast.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			this.panEast.Location = new System.Drawing.Point(544, 296);
			this.panEast.Size = new System.Drawing.Size(24, 24);
			this.panEast.Text = "E";
			// 
			// panWest
			// 
			this.panWest.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			this.panWest.Location = new System.Drawing.Point(496, 296);
			this.panWest.Size = new System.Drawing.Size(24, 24);
			this.panWest.Text = "W";
			// 
			// panSouth
			// 
			this.panSouth.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			this.panSouth.Location = new System.Drawing.Point(520, 304);
			this.panSouth.Size = new System.Drawing.Size(24, 24);
			this.panSouth.Text = "S";
			// 
			// panNorth
			// 
			this.panNorth.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			this.panNorth.Location = new System.Drawing.Point(520, 280);
			this.panNorth.Size = new System.Drawing.Size(24, 24);
			this.panNorth.Text = "N";
			// 
			// zoomOut
			// 
			this.zoomOut.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.75F, System.Drawing.FontStyle.Bold);
			this.zoomOut.Location = new System.Drawing.Point(448, 280);
			this.zoomOut.Size = new System.Drawing.Size(32, 24);
			this.zoomOut.Text = "out";
			// 
			// zoomIn
			// 
			this.zoomIn.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.75F, System.Drawing.FontStyle.Bold);
			this.zoomIn.Location = new System.Drawing.Point(448, 304);
			this.zoomIn.Size = new System.Drawing.Size(32, 24);
			this.zoomIn.Text = "in";
			// 
			// mapStatus
			// 
			this.mapStatus.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Regular);
			this.mapStatus.Location = new System.Drawing.Point(432, 224);
			this.mapStatus.Size = new System.Drawing.Size(128, 23);
			// 
			// label11
			// 
			this.label11.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label11.Location = new System.Drawing.Point(432, 200);
			this.label11.Size = new System.Drawing.Size(96, 24);
			this.label11.Text = "Status";
			// 
			// longitudeLabel
			// 
			this.longitudeLabel.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Regular);
			this.longitudeLabel.Location = new System.Drawing.Point(432, 128);
			this.longitudeLabel.Size = new System.Drawing.Size(128, 64);
			this.longitudeLabel.Text = "n/a";
			// 
			// latitudeLabel
			// 
			this.latitudeLabel.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Regular);
			this.latitudeLabel.Location = new System.Drawing.Point(432, 32);
			this.latitudeLabel.Size = new System.Drawing.Size(128, 72);
			this.latitudeLabel.Text = "n/a";
			// 
			// label7
			// 
			this.label7.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label7.Location = new System.Drawing.Point(432, 104);
			this.label7.Size = new System.Drawing.Size(96, 24);
			this.label7.Text = "Longitude";
			// 
			// label6
			// 
			this.label6.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, (System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline));
			this.label6.Location = new System.Drawing.Point(432, 8);
			this.label6.Size = new System.Drawing.Size(88, 24);
			this.label6.Text = "Latitude";
			// 
			// mapPicture
			// 
			this.mapPicture.Location = new System.Drawing.Point(8, 8);
			this.mapPicture.Size = new System.Drawing.Size(416, 320);
			// 
			// findTab
			// 
			this.findTab.Controls.Add(this.btnFind);
			this.findTab.Controls.Add(this.findState);
			this.findTab.Controls.Add(this.label3);
			this.findTab.Controls.Add(this.findCity);
			this.findTab.Controls.Add(this.label2);
			this.findTab.Controls.Add(this.findAddress);
			this.findTab.Controls.Add(this.label1);
			this.findTab.Location = new System.Drawing.Point(4, 22);
			this.findTab.Size = new System.Drawing.Size(572, 354);
			this.findTab.Text = "Find";
			// 
			// btnFind
			// 
			this.btnFind.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.btnFind.Location = new System.Drawing.Point(220, 120);
			this.btnFind.Size = new System.Drawing.Size(100, 36);
			this.btnFind.Text = "Show Map";
			this.btnFind.Click += new System.EventHandler(this.btnFind_Click);
			// 
			// findState
			// 
			this.findState.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.findState.Location = new System.Drawing.Point(120, 76);
			this.findState.Size = new System.Drawing.Size(212, 26);
			this.findState.Text = "";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label3.Location = new System.Drawing.Point(28, 80);
			this.label3.Size = new System.Drawing.Size(84, 24);
			this.label3.Text = "State:";
			this.label3.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// findCity
			// 
			this.findCity.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.findCity.Location = new System.Drawing.Point(120, 44);
			this.findCity.Size = new System.Drawing.Size(212, 26);
			this.findCity.Text = "";
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label2.Location = new System.Drawing.Point(28, 48);
			this.label2.Size = new System.Drawing.Size(84, 24);
			this.label2.Text = "City:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// findAddress
			// 
			this.findAddress.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.findAddress.Location = new System.Drawing.Point(120, 12);
			this.findAddress.Size = new System.Drawing.Size(212, 26);
			this.findAddress.Text = "";
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label1.Location = new System.Drawing.Point(28, 16);
			this.label1.Size = new System.Drawing.Size(84, 24);
			this.label1.Text = "Address:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// dataTab
			// 
			this.dataTab.Controls.Add(this.btnUploadData);
			this.dataTab.Controls.Add(this.btnClearData);
			this.dataTab.Controls.Add(this.label13);
			this.dataTab.Controls.Add(this.DataListView);
			this.dataTab.Location = new System.Drawing.Point(4, 22);
			this.dataTab.Size = new System.Drawing.Size(572, 354);
			this.dataTab.Text = "Data Store";
			// 
			// btnUploadData
			// 
			this.btnUploadData.Location = new System.Drawing.Point(396, 4);
			this.btnUploadData.Size = new System.Drawing.Size(80, 24);
			this.btnUploadData.Text = "Upload";
			this.btnUploadData.Click += new System.EventHandler(this.btnUploadData_Click);
			// 
			// btnClearData
			// 
			this.btnClearData.Location = new System.Drawing.Point(480, 4);
			this.btnClearData.Size = new System.Drawing.Size(80, 24);
			this.btnClearData.Text = "Clear";
			// 
			// label13
			// 
			this.label13.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Bold);
			this.label13.Location = new System.Drawing.Point(16, 8);
			this.label13.Size = new System.Drawing.Size(192, 24);
			this.label13.Text = "Local Data Store";
			// 
			// DataListView
			// 
			this.DataListView.Location = new System.Drawing.Point(8, 32);
			this.DataListView.Size = new System.Drawing.Size(560, 296);
			this.DataListView.View = System.Windows.Forms.View.Details;
			// 
			// optionsTab
			// 
			this.optionsTab.Controls.Add(this.btnApplyOptions);
			this.optionsTab.Controls.Add(this.label16);
			this.optionsTab.Controls.Add(this.label17);
			this.optionsTab.Controls.Add(this.txtServerInterval);
			this.optionsTab.Controls.Add(this.label15);
			this.optionsTab.Controls.Add(this.txtGPSInterval);
			this.optionsTab.Controls.Add(this.label14);
			this.optionsTab.Location = new System.Drawing.Point(4, 22);
			this.optionsTab.Size = new System.Drawing.Size(572, 354);
			this.optionsTab.Text = "Options";
			// 
			// btnApplyOptions
			// 
			this.btnApplyOptions.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.btnApplyOptions.Location = new System.Drawing.Point(268, 108);
			this.btnApplyOptions.Size = new System.Drawing.Size(100, 36);
			this.btnApplyOptions.Text = "Apply";
			this.btnApplyOptions.Click += new System.EventHandler(this.btnApplyOptions_Click);
			// 
			// label16
			// 
			this.label16.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label16.Location = new System.Drawing.Point(296, 68);
			this.label16.Size = new System.Drawing.Size(84, 24);
			this.label16.Text = "seconds";
			// 
			// label17
			// 
			this.label17.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label17.Location = new System.Drawing.Point(296, 24);
			this.label17.Size = new System.Drawing.Size(84, 24);
			this.label17.Text = "seconds";
			// 
			// txtServerInterval
			// 
			this.txtServerInterval.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.txtServerInterval.Location = new System.Drawing.Point(224, 64);
			this.txtServerInterval.Size = new System.Drawing.Size(68, 26);
			this.txtServerInterval.Text = "";
			// 
			// label15
			// 
			this.label15.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label15.Location = new System.Drawing.Point(20, 68);
			this.label15.Size = new System.Drawing.Size(200, 24);
			this.label15.Text = "Send data to server every";
			this.label15.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// txtGPSInterval
			// 
			this.txtGPSInterval.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.txtGPSInterval.Location = new System.Drawing.Point(224, 24);
			this.txtGPSInterval.Size = new System.Drawing.Size(68, 26);
			this.txtGPSInterval.Text = "";
			// 
			// label14
			// 
			this.label14.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular);
			this.label14.Location = new System.Drawing.Point(40, 28);
			this.label14.Size = new System.Drawing.Size(176, 24);
			this.label14.Text = "Get GPS data every";
			this.label14.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// MainForm
			// 
			this.ClientSize = new System.Drawing.Size(586, 383);
			this.Controls.Add(this.formTabs);
			this.Text = "Applied Data Systems Truck Tracking";

		}
		#endregion
		#endregion

		#region --- Form UI Events ---
		private void TabClick(object sender, System.EventArgs e)
		{
			switch(formTabs.SelectedIndex)
			{
				case 0:  // me tab
					break;
				case 1:  // map tab
					break;
				case 2:  // find tab
					RefreshCacheList();
					break;
				case 3:  // data tab
					break;
				case 4:  // history tab
					RefreshDataView();
					break;
				case 5:  // options tab
					break;
			}
			
		}

		private void PanZoom_Click(object sender, System.EventArgs e)
		{
			Button b = (Button)sender;
			float panchange = 0;

			if(b.Text.Equals("in"))
			{
				if(m_currentZoom > 0)
				{
					Debugging.DEBUGMSG(true, "Zoom In\n\r");
					m_currentZoom -= 1;
				}
			}
			else if(b.Text.Equals("out"))
			{
				if(m_currentZoom < 8)
				{
					Debugging.DEBUGMSG(true, "Zoom Out\n\r");
					m_currentZoom += 1;
				}
			}
			else
			{
				// determine pan amount
				switch(m_currentZoom)
				{
					case 0:
						panchange = 0.0005F;
						break;
					case 1:
						panchange = 0.001F;
						break;
					case 2:
						panchange = 0.0025F;
						break;
					case 3:
						panchange = 0.005F;
						break;
					case 4:
						panchange = 0.01F;
						break;
					case 5:
						panchange = 0.025F;
						break;
					case 6:
						panchange = 0.05F;
						break;
					case 7:
						panchange = 0.075F;
						break;
					case 8:
						panchange = 0.1F;
						break;
				}

				Debugging.DEBUGMSG(true, "Pan Value = " + panchange.ToString() + "\n\r");

				// pan
				if(b.Text.Equals("N"))
				{
					Debugging.DEBUGMSG(true, "Pan N\n\r");
					mapCenterLatitude += panchange;
				}
				else if(b.Text.Equals("S"))
				{
					Debugging.DEBUGMSG(true, "Pan S\n\r");
					mapCenterLatitude -= panchange;
				}
				else if(b.Text.Equals("E"))
				{
					Debugging.DEBUGMSG(true, "Pan E\n\r");
					mapCenterLongitude += panchange;
				}
				else if(b.Text.Equals("W"))
				{
					Debugging.DEBUGMSG(true, "Pan W\n\r");
					mapCenterLongitude -= panchange;
				}
			}
			
			pointLabel = "";

			RefreshMap(false);			
		}
		
		private void btnFind_Click(object sender, System.EventArgs e)
		{
			Debugging.DEBUGMSG(true, "Find Clicked\n\r");

			// show the map tab
			this.formTabs.SelectedIndex = 2;

			// find by address
			m_currentAddress.Street = findAddress.Text;
			m_currentAddress.City = findCity.Text;
			m_currentAddress.State = findState.Text;
			m_currentAddress.Latitude = 0;
			m_currentAddress.Longitude = 0;
			m_currentAddress.Geocoded = false;
			pointLabel = findAddress.Text;
			
			mapStatus.Text = "Geocoding...";
			Debugging.DEBUGMSG(true, "Geocoding...\n\r");

			this.Refresh();

			cartographer.GeocodeAddress(ref m_currentAddress);
			mapCenterLatitude = m_currentAddress.Latitude;
			mapCenterLongitude = m_currentAddress.Longitude;

			cartographer.LastLocation.Label = m_currentAddress.Street;
			cartographer.LastLocation.IconDataSource = "MapPoint.Icons";
			cartographer.LastLocation.IconName = "0";
			cartographer.LastLocation.PinID = "Find";
			cartographer.LastLocation.LatLong.Latitude = m_currentAddress.Latitude;
			cartographer.LastLocation.LatLong.Longitude = m_currentAddress.Longitude;

			RefreshMap(true);
		}

		private void btnShowCached_Click(object sender, System.EventArgs e)
		{
			cartographer.DisplayMapImage(cartographer.cachePath + "\\" + cacheList.Items[cacheList.SelectedIndices[0]].Text, mapPicture, true);
			mapCenterLatitude = cartographer.LastLocation.LatLong.Latitude;
			mapCenterLongitude = cartographer.LastLocation.LatLong.Longitude;

			formTabs.SelectedIndex = 2;
		}

		private void btnClearCache_Click(object sender, System.EventArgs e)
		{
			try
			{
				string[] files = Directory.GetFiles(cartographer.cachePath);

				foreach(string file in files)
					File.Delete(file);
			}
			catch(IOException)
			{
				// nothing to delete
			}

			RefreshCacheList();
		}

		private void btnClearChecked_Click(object sender, System.EventArgs e)
		{
			foreach(ListViewItem lvi in cacheList.Items)
			{
				if(lvi.Checked)
					File.Delete(cartographer.cachePath + "\\" + lvi.Text + ".map");
			}
			
			RefreshCacheList();
		}

		private void chkSimulateGPS_CheckStateChanged(object sender, System.EventArgs e)
		{
			// enable/disable the simulation timer
			CheckBox cb = (CheckBox)sender;

			if(cb.Checked)
			{
				simulationData.MoveFirst();
			}

			simulationTimer.Enabled = cb.Checked;
		}

		private void btnUploadData_Click(object sender, System.EventArgs e)
		{
			m_vehicleData.Upload();
			RefreshDataView();
		}

		private void btnApplyOptions_Click(object sender, System.EventArgs e)
		{
			try
			{
				// set the gps read receive timer
				GPSReadInterval = int.Parse(txtGPSInterval.Text) * 1000;
			}
			catch(Exception)
			{
				// likely invalid textbox data
				txtGPSInterval.Text = GPSReadInterval.ToString();
			}

			try
			{
				// set the gps read receive timer
				DataSendInterval = int.Parse(txtServerInterval.Text) * 1000;
			}
			catch(Exception)
			{
				// likely invalid textbox data
				txtServerInterval.Text = DataSendInterval.ToString();
			}
		}
		#endregion

		#region --- Timer Events ---
		private void simulationTimer_Tick(object sender, System.EventArgs e)
		{
			// simulate GPS receiver data
			if(simulationForward)
			{
				simulationData.MoveNext();

				if(simulationData.EOF())
				{
					simulationForward = false;
				}
			}
			else
			{
				simulationData.MovePrevious();

				if(simulationData.BOF())
				{
					simulationForward = true;
				}
			}
			
			CurrentLatitude = simulationData.Coordinate.Latitude;
			CurrentLongitude = simulationData.Coordinate.Longitude;

			SaveDataPoint();
		}

		private void dataTimer_Tick(object sender, System.EventArgs e)
		{
			// attempt to upload data to server
			lstActivity.Items.Insert(0, "Sending " + m_vehicleData.Data.Rows.Count + " rows to server");
			
			Thread uploadThread = new Thread(new ThreadStart(m_vehicleData.Upload));
			uploadThread.Start();

			RefreshDataView();
		}

		#endregion

		#region --- private methods ---
		private void SaveDataPoint()
		{
			if(!m_vehicleData.WriteRecord(CurrentLatitude, CurrentLongitude, GetCurrentSpeed()))
			{
				// the write failed
				if(chkSimulateGPS.Checked)
				{
					simulationTimer.Enabled = false;
					chkSimulateGPS.Checked = false;

					MessageBox.Show("Error in WriteRecord");
				}
			}
			else
			{
				RefreshDataView();
			}
		}

		private short GetCurrentSpeed()
		{
			// this method would query some hardware or vehicle computer for speed.
			// we'll simulate it
			Random rand = new Random();

			return (short)rand.Next(75);
		}

		private void RefreshDataView()
		{
			DataListView.Items.Clear();
			DataListView.Columns.Clear();

			foreach(DataColumn dc in m_vehicleData.Data.Columns)
			{
				DataListView.Columns.Add(dc.ToString(), 100, HorizontalAlignment.Center);
			}

			foreach(DataRow dr in m_vehicleData.Data.Rows)
			{
				string[] items = new string[m_vehicleData.Data.Columns.Count];

				ListViewItem lvi = new ListViewItem(dr[0].ToString());

				for(short index = 1 ; index < m_vehicleData.Data.Columns.Count ; index++)
				{
					lvi.SubItems.Add(dr[index].ToString());
				}

				DataListView.Items.Add(lvi);				
			}
		}

		private void RefreshCacheList()
		{
			cacheList.Items.Clear();

			try
			{
				string[] files = Directory.GetFiles(cartographer.cachePath, "*.map");

				foreach(string file in files)
				{
					if(!file.Equals("temp,map"))
						cacheList.Items.Add(new ListViewItem(file.Replace(cartographer.cachePath + "\\", "")));
				}
			}
			catch(IOException)
			{
				cacheList.Items.Add(new ListViewItem("<cache empty>"));
			}			
		}

		private void RefreshMap(bool cacheMap)
		{
			Tools.WaitCursorOn();
			Debugging.DEBUGMSG(true, "Refreshing map...\n\r");

			mapStatus.Text = "Getting Map...";
			this.Refresh();
			
			// geocode if necessary
			if(!m_currentAddress.Geocoded)
			{
				Debugging.DEBUGMSG(true, "Geocoding Address\n\r");
				cartographer.GeocodeAddress(ref m_currentAddress);

				mapCenterLatitude = m_currentAddress.Latitude;
				mapCenterLongitude = m_currentAddress.Longitude;
			}
			
				
			Address distanceAddress = new Address();
			distanceAddress.Latitude = CurrentLatitude;
			distanceAddress.Longitude = CurrentLongitude;
			double distance = cartographer.DistanceInMiles(m_currentAddress, distanceAddress);	
			
			if(distance > 0)
			{
				lblDistance.Text = distance.ToString("0.00");
			}
			else
			{
				lblDistance.Text = "unknown";
			}

			// show the map
			try
			{
				Debugging.DEBUGMSG(true, "Display Map\n\r");
				int zoom = 0;

				switch(m_currentZoom)
				{
					case 0:
						zoom = 5000;
						break;
					case 1:
						zoom = 10000;
						break;
					case 2:
						zoom = 25000;
						break;
					case 3:
						zoom = 50000;
						break;
					case 4:
						zoom = 100000;
						break;
					case 5:
						zoom = 250000;
						break;
					case 6:
						zoom = 500000;
						break;
					case 7:
						zoom = 750000;
						break;
					case 8:
						zoom = 10000000;
						break;
				}

				try
				{
					cartographer.DisplayMapImage(mapCenterLatitude, mapCenterLongitude, zoom, this.mapPicture, cacheMap);
					
					Debugging.DEBUGMSG(true, "Map displayed.\n\r");

					// display the lat & lon for the address
					latitudeLabel.Text = m_currentAddress.Coordinates.Latitude.Degrees + " \n" + m_currentAddress.Coordinates.Latitude.Minutes + " '\n" + m_currentAddress.Coordinates.Latitude.Seconds + " \"\n" + m_currentAddress.Coordinates.Latitude.Direction;
					longitudeLabel.Text = m_currentAddress.Coordinates.Longitude.Degrees + " \n" + m_currentAddress.Coordinates.Longitude.Minutes + " '\n" + m_currentAddress.Coordinates.Longitude.Seconds + " \"\n" + m_currentAddress.Coordinates.Longitude.Direction;

					// show the map tab
					this.formTabs.SelectedIndex = 2;

					// save the lat/lon
					CurrentLatitude = mapCenterLatitude;
					CurrentLongitude = mapCenterLongitude;
				}
				catch(Exception ex)
				{
					Debugging.DEBUGMSG(true, "Failed to display map: " + ex.Message + "\n\r");
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
			
			Tools.WaitCursorOff();
			mapStatus.Text = "Ready...";
		}

		private string GetVIN()
		{
			// simulate retrieving vehicle VIN
			Random rnd = new Random();
			int newID = rnd.Next(1, 25);
			
			lstActivity.Items.Insert(0, "VIN = ADS-DC-0" + newID.ToString());

			return "ADS-DC-0" + newID.ToString();
		}
		#endregion

	}
}
