using System;

namespace TruckTrack
{
	/// <summary>
	/// 
	/// </summary>
	public struct Coordinate
	{
		double m_lat;
		double m_lon;
	
		public Coordinate(double Latitude, double Longitude)
		{
			m_lat = Latitude;
			m_lon = Longitude;
		}
	
		public double Latitude
		{
			get
			{
				return m_lat;
			}
			set
			{
				m_lat = value;
			}
		}

		public double Longitude
		{
			get
			{
				return m_lon;
			}
			set
			{
				m_lon = value;
			}
		}
	}


	public class GPSRoute : System.Collections.CollectionBase
	{
		private int m_index = 0;
		
		public GPSRoute()
		{
		}

		public int Index
		{
			get
			{
				return m_index;
			}
			set
			{
				if((value > (List.Count - 1)) || (value < 0))
				{
					throw new Exception("Invalid Index");
				}
				else
				{
					m_index = value;
				}
			}
		}

		public Coordinate Coordinate
		{
			get
			{
				return (Coordinate)List[m_index];
			}
		}

		public void Add(Coordinate Coordinate)
		{
			List.Add(Coordinate);
		}

		public void MoveNext()
		{
			if(m_index < (List.Count - 1))
			{
				m_index++;
			}
		}

		public void MovePrevious()
		{
			if (m_index >= 0)
			{
				m_index--;
			}
		}

		public void MoveFirst()
		{
			m_index = 0;
		}

		public void MoveLast()
		{
			m_index = List.Count - 1;
		}

		public bool EOF()
		{
			if(m_index >= (List.Count - 1))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		public bool BOF()
		{
			if(m_index <= 0)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		public Coordinate this[int Index]
		{
			get
			{
				return (Coordinate)List[Index];
			}
			set
			{
				List[Index] = value;
			}
		}
	}


	public class ADSToBaltimore : GPSRoute
	{
		private Coordinate[] m_data = 
			{  // crummy straight-line data, but still a sample
				new Coordinate(39.1694619170274, -76.8455437011529), // ADS office
				new Coordinate(39.1794619170274, -76.8255437011529),
				new Coordinate(39.1894619170274, -76.8055437011529),
				new Coordinate(39.1994619170274, -76.7855437011529),
				new Coordinate(39.2094619170274, -76.7655437011529),
				new Coordinate(39.2194619170274, -76.7455437011529),
				new Coordinate(39.2294619170274, -76.7255437011529),
				new Coordinate(39.2394619170274, -76.7055437011529),
				new Coordinate(39.2494619170274, -76.6855437011529),
				new Coordinate(39.2594619170274, -76.6655437011529),
				new Coordinate(39.2694619170274, -76.6455437011529),
				new Coordinate(39.2794619170274, -76.6255437011529),
				new Coordinate(39.2864144214533, -76.6191716204704)  // 300 W Pratt in Balto
			};

		public ADSToBaltimore()
		{
			foreach(Coordinate c in m_data)
			{
				this.Add(c);
			}
		}
	}
}
