using System;
using System.Data;
using System.Web;
using ADS;
using TruckWeb;

namespace TruckTrack
{
	/// <summary>
	/// 
	/// </summary>
	public class VehicleData
	{
		// ensure the data is saved to and loaded from the app directory
		public string m_fileName = System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase.Replace(".exe", ".xml");

		string m_vehicleID;
		DataSet m_data;
		
		// expose read-only DataTable
		public DataTable Data
		{
			get
			{
				return m_data.Tables["VehicleData"];
			}
		}

		public VehicleData(string VehicleID)
		{
			m_vehicleID = VehicleID;

			try
			{
				m_data = new DataSet();
			
				DataTable dt = m_data.Tables.Add("VehicleData");

				dt.Columns.Add("VehicleID", Type.GetType("System.String"));		// VehicleID
				dt.Columns.Add("RecordDate", Type.GetType("System.DateTime"));	// RecordDate
				dt.Columns.Add("Latitude", Type.GetType("System.Double"));		// Latitude
				dt.Columns.Add("Longitude", Type.GetType("System.Double"));		// Longitude
				dt.Columns.Add("Speed", Type.GetType("System.Int16"));		// Longitude
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "Exception creating VehicleData: " + ex.Message + "\r\n");
			}
		}

		// data writes MUST go through this accessor - allows control to ensure only
		// valid data will get passed to the Web Service
		public bool WriteRecord(double Latitude, double Longitude, Int16 Speed)
		{
			try
			{				
				DataTable dt = m_data.Tables["VehicleData"];
				DataRow row = dt.NewRow();
				
				row["VehicleID"] = m_vehicleID;
				row["RecordDate"] = DateTime.Now;
				row["Latitude"] = Latitude;
				row["Longitude"] = Longitude;
				row["Speed"] = Speed;
				
				dt.Rows.Add(row);

				dt.AcceptChanges();
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "Exception in WriteRecord: " + ex.Message + "\r\n");
				return false;
			}

			return true;
		}

		public void Clear()
		{
			m_data.Clear();
		}

		public void Load()
		{
			try
			{
				Debugging.DEBUGMSG(true, "loading XML data from '" + m_fileName + "'\r\n");
				m_data.ReadXml(m_fileName);
			}
			catch(Exception)
			{
				Debugging.DEBUGMSG(true, "no data file found\r\n");
				// likely a file not found, so we'll ignore it
			}
		}

		public void Save()
		{
			Debugging.DEBUGMSG(true, "saving XML data to '" + m_fileName + "'\r\n");
			m_data.WriteXml(m_fileName);
		}
	
		public void Upload()
		{
			try
			{
				// upload to remote server's Web Service
				DataService ds = new DataService();

				// send the data row by row
				// only remove a local row when confirmation is given of an insert on
				// the server to help maintain data integrity.  the remote DB SP prevents 
				// data duplication, so we've essentially implemented a queue
				Debugging.DEBUGMSG(true, "Uploading Data...\r\n");
				
				while(m_data.Tables["VehicleData"].Rows.Count > 0)
				{
					if(ds.AddDataPoint
						(
						m_data.Tables["VehicleData"].Rows[0]["VehicleID"].ToString(),
						(DateTime)m_data.Tables["VehicleData"].Rows[0]["RecordDate"],
						(double)m_data.Tables["VehicleData"].Rows[0]["Latitude"],
						(double)m_data.Tables["VehicleData"].Rows[0]["Longitude"],
						(short)m_data.Tables["VehicleData"].Rows[0]["Speed"]
						)
						)
					{
						m_data.Tables["VehicleData"].Rows.RemoveAt(0);
					}
				}
				Debugging.DEBUGMSG(true, "Finished uploading.\r\n");
			}
			catch(Exception ex)
			{
				Debugging.DEBUGMSG(true, "Exception uploading data: " + ex.Message + "\r\n");
			}
		}
	}
}
